@@ -543,7 +543,7 @@ int mutt_view_attachment (FILE *fp, BODY *a, int flag, HEADER *hdr,
       /* Use built-in handler */
       set_option (OPTVIEWATTACH); /* disable the "use 'v' to view this part"
 				   * message in case of error */
-      if (mutt_decode_save_attachment (fp, a, pagerfile, 1, 0))
+      if (mutt_decode_save_attachment (fp, a, pagerfile, M_DISPLAY, 0))
       {
 	unset_option (OPTVIEWATTACH);
 	goto return_error;
@@ -801,7 +801,7 @@ int mutt_decode_save_attachment (FILE *fp, BODY *m, char *path,
   HEADER *saved_hdr = NULL;
 
   memset (&s, 0, sizeof (s));
-  s.flags = (displaying ? M_DISPLAY : 0);
+  s.flags = displaying;
 
   if (flags == M_SAVE_APPEND)
     s.fpout = fopen (path, "a");
@@ -896,6 +896,8 @@ int mutt_print_attachment (FILE *fp, BODY *a)
     rfc1524_entry *entry;
     int piped = FALSE;
 
+    dprint (2, (debugfile, "Using mailcap...\n"));
+    
     entry = rfc1524_new_entry ();
     rfc1524_mailcap_lookup (a, type, entry, M_PRINT);
     if (rfc1524_expand_filename (entry->nametemplate, a->filename,
@@ -964,8 +966,8 @@ int mutt_print_attachment (FILE *fp, BODY *a)
     return (1);
   }
 
-  if (!ascii_strcasecmp ("text/plain", a->subtype) ||
-      !ascii_strcasecmp ("application/postscript", a->subtype))
+  if (!ascii_strcasecmp ("text/plain", type) ||
+      !ascii_strcasecmp ("application/postscript", type))
   {
     return (mutt_pipe_attachment (fp, a, NONULL(PrintCmd), NULL));
   }
@@ -979,13 +981,19 @@ int mutt_print_attachment (FILE *fp, BODY *a)
     fpout = NULL;
     
     mutt_mktemp (newfile);
-    if (mutt_decode_save_attachment (fp, a, newfile, 0, 0) == 0)
+    if (mutt_decode_save_attachment (fp, a, newfile, M_PRINTING, 0) == 0)
     {
+      
+      dprint (2, (debugfile, "successfully decoded %s type attachment to %s\n",
+		  type, newfile));
+      
       if ((ifp = fopen (newfile, "r")) == NULL)
       {
 	mutt_perror ("fopen");
 	goto bail0;
       }
+
+      dprint (2, (debugfile, "successfully opened %s read-only\n", newfile));
       
       mutt_endwin (NULL);
       if ((thepid = mutt_create_filter (NONULL(PrintCmd), &fpout, NULL, NULL)) < 0)
@@ -994,7 +1002,13 @@ int mutt_print_attachment (FILE *fp, BODY *a)
 	goto bail0;
       }
 
+      dprint (2, (debugfile, "Filter created.\n"));
+      
       mutt_copy_stream (ifp, fpout);
+
+      safe_fclose (&fpout);
+      safe_fclose (&ifp);
+
       if (mutt_wait_filter (thepid) != 0 || option (OPTWAITKEY))
 	mutt_any_key_to_continue (NULL);
       rc = 1;
