@@ -989,6 +989,11 @@ static void text_plain_flowed_handler (BODY *a, STATE *s)
   
   if ((flowed_max = FLOWED_MAX) > COLS - 3)
     flowed_max = COLS - 3;
+  if (flowed_max > COLS - WrapMargin)
+    flowed_max = COLS - WrapMargin;
+  if (flowed_max <= 0)
+    flowed_max = COLS;
+    
 
   while (bytes > 0 && fgets (line, sizeof (line), s->fpin))
   {
@@ -1669,8 +1674,8 @@ static void external_body_handler (BODY *b, STATE *s)
       state_mark_attach (s);
       state_printf (s, _("[-- This %s/%s attachment is not included, --]\n"),
 		    TYPE(b->parts), b->parts->subtype);
-      state_attach_puts (_("[-- and the indicated external source has --]\n"), s);
-      state_attach_puts (_("[-- expired. --]\n"), s);
+      state_attach_puts (_("[-- and the indicated external source has --]\n"
+			   "[-- expired. --]\n"), s);
 
       mutt_copy_hdr(s->fpin, s->fpout, ftell (s->fpin), b->parts->offset,
 		    (option (OPTWEED) ? (CH_WEED | CH_REORDER) : 0) |
