@@ -104,14 +104,26 @@ int imap_create_mailbox (IMAP_DATA* idata, char* mailbox)
   return 0;
 }
 
-int imap_delete_mailbox (CONTEXT* ctx, char* mailbox)
+int imap_delete_mailbox (CONTEXT* ctx, IMAP_MBOX mx)
 {
   char buf[LONG_STRING], mbox[LONG_STRING];
-  
-  imap_munge_mbox_name (mbox, sizeof (mbox), mailbox);
+  IMAP_DATA *idata;
+
+  if (!ctx || !ctx->data) {
+  	if (!(idata = imap_conn_find (&mx.account,
+    		option (OPTIMAPPASSIVE) ? M_IMAP_CONN_NONEW : 0)))
+  	{
+    		FREE (&mx.mbox);
+    		return -1;
+	}
+  } else {
+	  idata = ctx->data;
+  }
+
+  imap_munge_mbox_name (mbox, sizeof (mbox), mx.mbox);
   snprintf (buf, sizeof (buf), "DELETE %s", mbox);
 
-  if (imap_exec ((IMAP_DATA*) ctx->data, buf, 0) != 0)
+  if (imap_exec ((IMAP_DATA*) idata, buf, 0) != 0)
     return -1;
 
   return 0;
@@ -1115,10 +1127,10 @@ int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
    * changes to process, since we can reopen here. */
   imap_cmd_finish (idata);
 
-  if (idata->check_status & IMAP_NEWMAIL_PENDING)
-    result = M_NEW_MAIL;
-  else if (idata->check_status & IMAP_EXPUNGE_PENDING)
+  if (idata->check_status & IMAP_EXPUNGE_PENDING)
     result = M_REOPENED;
+  else if (idata->check_status & IMAP_NEWMAIL_PENDING)
+    result = M_NEW_MAIL;
   else if (idata->check_status & IMAP_FLAGS_PENDING)
     result = M_FLAGS;
 
