@@ -226,7 +226,14 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
 
       /* if we got output, make a new string consiting of the shell ouptput
 	 plus whatever else was left on the original line */
-      if (expn.data)
+      /* BUT: If this is inside a quoted string, directly add output to 
+       * the token */
+      if (expn.data && qc)
+      {
+	mutt_buffer_addstr (dest, expn.data);
+	FREE (&expn.data);
+      }
+      else if (expn.data)
       {
 	expnlen = mutt_strlen (expn.data);
 	tok->dsize = expnlen + mutt_strlen (tok->dptr) + 1;
@@ -428,27 +435,40 @@ static int parse_unalias (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *er
   {
     mutt_extract_token (buf, s, 0);
 
-    for (tmp = Aliases; tmp; tmp = tmp->next)
+    if (mutt_strcmp ("*", buf->data) == 0)
     {
-      if (mutt_strcasecmp (buf->data, tmp->name) == 0)
+      if (CurrentMenu == MENU_ALIAS)
       {
-	if (CurrentMenu == MENU_ALIAS)
-	{
+	for (tmp = Aliases; tmp ; tmp = tmp->next)
 	  tmp->del = 1;
-	  set_option (OPTFORCEREDRAWINDEX);
+	set_option (OPTFORCEREDRAWINDEX);
+      }
+      else
+	mutt_free_alias (&Aliases);
+      break;
+    }
+    else
+      for (tmp = Aliases; tmp; tmp = tmp->next)
+      {
+	if (mutt_strcasecmp (buf->data, tmp->name) == 0)
+	{
+	  if (CurrentMenu == MENU_ALIAS)
+	  {
+	    tmp->del = 1;
+	    set_option (OPTFORCEREDRAWINDEX);
+	    break;
+	  }
+
+	  if (last)
+	    last->next = tmp->next;
+	  else
+	    Aliases = tmp->next;
+	  tmp->next = NULL;
+	  mutt_free_alias (&tmp);
 	  break;
 	}
-
-	if (last)
-	  last->next = tmp->next;
-	else
-	  Aliases = tmp->next;
-	tmp->next = NULL;
-	mutt_free_alias (&tmp);
-	break;
+	last = tmp;
       }
-      last = tmp;
-    }
   }
   while (MoreArgs (s));
   return 0;
@@ -747,6 +767,8 @@ static void mutt_restore_default (struct option_t *p)
     set_option (OPTSORTSUBTHREADS);
   if (p->flags & R_RESORT)
     set_option (OPTNEEDRESORT);
+  if (p->flags & R_RESORT_INIT)
+    set_option (OPTRESORTINIT);
 }
 
 static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
@@ -1190,6 +1212,8 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
       set_option (OPTSORTSUBTHREADS);
     if (MuttVars[idx].flags & R_RESORT)
       set_option (OPTNEEDRESORT);
+    if (MuttVars[idx].flags & R_RESORT_INIT)
+      set_option (OPTRESORTINIT);
   }
   return (r);
 }
@@ -1682,6 +1706,12 @@ void mutt_init (int skip_sys_rc, LIST *commands)
   err.data = error;
   err.dsize = sizeof (error);
 
+  /* 
+   * XXX - use something even more difficult to predict?
+   */
+  snprintf (AttachmentMarker, sizeof (AttachmentMarker),
+	    "\033]9;%ld\a", (long) time (NULL));
+  
   /* on one of the systems I use, getcwd() does not return the same prefix
      as is listed in the passwd file */
   if ((p = getenv ("HOME")))
