@@ -51,6 +51,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 
 #define ISHEADER(x) ((x) == MT_COLOR_HEADER || (x) == MT_COLOR_HDEFAULT)
 
@@ -1056,6 +1057,8 @@ static int format_line (struct line_t **lineInfo, int n, unsigned char *buf,
     k = mbrtowc (&wc, (char *)buf+ch, cnt-ch, &mbstate);
     if (k == -2 || k == -1)
     {
+      dprint (1, (debugfile, "%s:%d: mbrtowc returned %d; errno = %d.\n",
+		  __FILE__, __LINE__, k, errno));
       if (col + 4 > wrap_cols)
 	break;
       col += 4;
@@ -2365,7 +2368,7 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 	break;
 
       case OP_FORWARD_MESSAGE:
-	CHECK_MODE(IsHeader (extra) && !IsMsgAttach (extra));
+	CHECK_MODE(IsHeader (extra) || IsMsgAttach (extra));
         CHECK_ATTACH;
         if (IsMsgAttach (extra))
 	  mutt_attach_forward (extra->fp, extra->hdr, extra->idx,
